﻿Module DrawBitmap

    Friend UpdateImage As Boolean = True

    Friend SelectedIndex As Int32 = -1

    Friend WireDiam As Single
    Friend NeedleDiam As Single

    Friend StopAngle As Int32 = 90
    Friend StartAngle As Int32 = 270

    Friend PixelPerMM As Single = 10
    Friend StartX As Single
    Friend StartY As Single

    Friend SewingAreaMaxY As Int32
    Friend SewingAreaMinY As Int32
    Friend SewingAreaMaxX As Int32
    Friend SewingAreaMinX As Int32

    Friend FlagScaleText As Boolean

    Private TextFont As Font = New Font("Arial", 11, FontStyle.Bold)
    Private TextBrush As SolidBrush = New SolidBrush(Color.FromArgb(120, 120, 0))

    Private PenSewingArea As Pen = New Pen(Color.FromArgb(150, 150, 0), 4)

    'Private PenScale1 As Pen = New Pen(Color.FromArgb(240, 220, 220))
    'Private PenScale5 As Pen = New Pen(Color.FromArgb(210, 200, 200))
    'Private PenScale10 As Pen = New Pen(Color.FromArgb(170, 150, 150))

    Private PenScale1 As Pen = New Pen(Color.FromArgb(220, 200, 200))
    Private PenScale5 As Pen = New Pen(Color.FromArgb(180, 170, 170))
    Private PenScale10 As Pen = New Pen(Color.FromArgb(150, 130, 130))

    Private StartColor As Color = Color.FromArgb(255, 0, 0)
    Private WireColor As Color = Color.FromArgb(80, 0, 150, 0)
    Private NeedleColor As Color = Color.Black
    Private SelectionColor As Color = Color.Aquamarine
    Private BackColor As Color = Color.FromArgb(250, 250, 250)

    Private StartPen As Pen
    Private StartBrush As SolidBrush
    Private WirePen As Pen
    Private WireBrush As SolidBrush
    Private NeedlePen As Pen
    Private NeedleBrush As SolidBrush

    Private SelectionPen As Pen
    Private SelectionBrush As SolidBrush

    Private GFX As Graphics
    Private BMP As Bitmap
    Private BmpW As Int32
    Private BmpH As Int32

    ' ========================================================================================================
    '   RESIZE DIMENSIONS - RESET DIMENSIONS and ZOOM
    ' ========================================================================================================
    Friend Sub ResizeDimensions(ByVal pbox As PictureBox)
        ' ---------------------------------------------------------------
        If pbox.ClientSize.Width < 1 Or pbox.ClientSize.Height < 1 Then Return
        If BmpW < 1 Or BmpH < 1 Then
            InitImage(pbox)
        End If
        ' ---------------------------------------------------------------
        Dim kx As Double = pbox.ClientSize.Width / BmpW
        Dim ky As Double = pbox.ClientSize.Height / BmpH
        Dim k As Double = Math.Sqrt(kx * ky)
        StartX = CInt(StartX * k)
        StartY = CInt(StartY * k)
        PixelPerMM = CSng(PixelPerMM * k)
        ' ---------------------------------------------------------------
        DrawImage(pbox)
        pbox.Refresh()
    End Sub

    Friend Sub ResetDimensions()
        Dim maxx As Double = BmpW / (SewingAreaMaxX - SewingAreaMinX)
        Dim maxy As Double = BmpH / (SewingAreaMaxY - SewingAreaMinY)
        If maxx < maxy Then
            PixelPerMM = CSng(maxx * 0.9)
        Else
            PixelPerMM = CSng(maxy * 0.9)
        End If
        Dim xp As Double = PixelPerMM * (SewingAreaMaxX - SewingAreaMinX)
        Dim yp As Double = PixelPerMM * (SewingAreaMaxY - SewingAreaMinY)
        StartX = CInt(-(BmpW - xp) / 2)
        StartY = CInt(-(BmpH - yp) / 2)
        InitDrawingParams()
    End Sub

    Friend Sub ChangeZoom(ByVal delta As Single, ByVal mousex As Int32, ByVal mousey As Int32)
        '
        ' ------------------------------------------- calc the mm-values before changing the zoom
        Dim Xmm As Single = PixelXtoMM(mousex)
        Dim Ymm As Single = PixelYtoMM(mousey)
        ' ------------------------------------------- change the zoom
        PixelPerMM *= (1 + delta)
        If PixelPerMM > 100 Then PixelPerMM = 100
        If PixelPerMM < 2 Then PixelPerMM = 2
        ' ------------------------------------------- pixels of to the mm-values after the zoom
        Dim Xpix As Single = MMtoPixelX(Xmm)
        Dim Ypix As Single = MMtoPixelY(Ymm)
        ' ------------------------------------------- the mm values must be in the old pixel position
        StartX = Xmm * PixelPerMM - mousex
        StartY = PixelPerMM * Ymm - BmpH + mousey
        ' ------------------------------------------- redraw
        InitDrawingParams()
        UpdateImage = True
    End Sub


    ' ========================================================================================================
    '   SCALE FUNCTIONS
    ' ========================================================================================================
    Friend Function PixelXtoMM(ByVal x As Single) As Single
        Return (x + StartX) / PixelPerMM
    End Function
    Friend Function PixelYtoMM(ByVal y As Single) As Single
        Return (BmpH + StartY - y) / PixelPerMM
    End Function
    Friend Function MMtoPixelX(ByVal mm As Single) As Single
        Return mm * PixelPerMM - StartX
    End Function
    Friend Function MMtoPixelY(ByVal mm As Single) As Single
        Return BmpH - PixelPerMM * mm + StartY
    End Function


    ' ========================================================================================================
    '   DRAWING
    ' ========================================================================================================
    Friend Sub InitDrawing()
        SelectedIndex = -1
        DataArray1.Flush()
        SW1.Reset()
        SW1.Start()
    End Sub

    Friend Sub InitDrawingParams()
        StartPen = New Pen(StartColor, 4 * WireDiam * PixelPerMM)
        StartBrush = New SolidBrush(StartColor)
        WirePen = New Pen(WireColor, WireDiam * PixelPerMM)
        WireBrush = New SolidBrush(WireColor)
        NeedlePen = New Pen(NeedleColor, WireDiam * PixelPerMM)
        NeedleBrush = New SolidBrush(NeedleColor)
        SelectionPen = New Pen(SelectionColor, WireDiam * PixelPerMM)
        SelectionBrush = New SolidBrush(SelectionColor)
    End Sub

    Friend Sub InitImage(ByVal pbox As PictureBox)
        If pbox.ClientSize.Width <> BmpW Or pbox.ClientSize.Height <> BmpH Then
            If pbox.ClientSize.Width < 1 Or pbox.ClientSize.Height < 1 Then Return
            BmpW = pbox.ClientSize.Width
            BmpH = pbox.ClientSize.Height
            ' ----------------------------------------------------------
            BMP = New Bitmap(BmpW, BmpH)
            Form1.Pbox1.Image = BMP
            GFX = Graphics.FromImage(Form1.Pbox1.Image)
            GFX.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
            GFX.Clear(BackColor)
        End If
    End Sub

    Friend Function IsNeedleUp(ByVal Zangle As Single) As Boolean
        If Zangle < StopAngle Or Zangle > StartAngle Then
            Return True
        Else
            Return False
        End If
    End Function

    Friend Sub DrawScale()
        Dim x1 As Int32 = CInt(Math.Floor(PixelXtoMM(0)))
        Dim y1 As Int32 = CInt(Math.Floor(PixelYtoMM(BmpH)))
        Dim x2 As Int32 = CInt(Math.Ceiling(PixelXtoMM(BmpW)))
        Dim y2 As Int32 = CInt(Math.Ceiling(PixelYtoMM(0)))
        Dim px, py As Single
        ' ---------------------------------------------------------- PAPER
        For x As Int32 = x1 To x2
            px = MMtoPixelX(x)
            If x Mod 10 = 0 Then
                If PixelPerMM < 3 Then
                    GFX.DrawLine(PenScale5, px, 0, px, BmpH)
                Else
                    GFX.DrawLine(penscale10, px, 0, px, BmpH)
                End If
            ElseIf x Mod 5 = 0 And PixelPerMM > 3 Then
                GFX.DrawLine(PenScale5, px, 0, px, BmpH)
            ElseIf PixelPerMM > 6 Then
                GFX.DrawLine(PenScale1, px, 0, px, BmpH)
            End If
        Next
        For y As Int32 = y1 To y2
            py = MMtoPixelY(y)
            If y Mod 10 = 0 Then
                If PixelPerMM < 3 Then
                    GFX.DrawLine(PenScale5, 0, py, BmpW, py)
                Else
                    GFX.DrawLine(penscale10, 0, py, BmpW, py)
                End If
            ElseIf y Mod 5 = 0 And PixelPerMM > 3 Then
                GFX.DrawLine(PenScale5, 0, py, BmpW, py)
            ElseIf PixelPerMM > 6 Then
                GFX.DrawLine(PenScale1, 0, py, BmpW, py)
            End If
        Next
        ' ---------------------------------------------------------- RECTANGLE
        Dim yy2 As Int32 = CInt(MMtoPixelY(SewingAreaMaxY))
        Dim yy1 As Int32 = CInt(MMtoPixelY(SewingAreaMinY))
        Dim xx2 As Int32 = CInt(MMtoPixelX(SewingAreaMaxX))
        Dim xx1 As Int32 = CInt(MMtoPixelX(SewingAreaMinX))
        GFX.DrawRectangle(PenSewingArea, _
                          xx1, yy2, _
                          xx2 - xx1, yy1 - yy2)
        ' ---------------------------------------------------------- TEXT
        If FlagScaleText Then
            If PixelPerMM >= 3 Then
                For x As Int32 = x1 To x2
                    px = MMtoPixelX(x)
                    If x Mod 10 = 0 Then
                        If x >= SewingAreaMinX And x <= SewingAreaMaxX Then
                            GFX.DrawString(x.ToString, TextFont, TextBrush, px, BmpH - 18)
                        End If
                    End If
                Next
                For y As Int32 = y1 To y2
                    py = MMtoPixelY(y)
                    If y Mod 10 = 0 Then
                        If y >= SewingAreaMinY And y <= SewingAreaMaxY Then
                            GFX.DrawString(y.ToString, TextFont, TextBrush, 2, py - 16)
                        End If
                    End If
                Next
            End If
        End If
    End Sub

    Friend Sub DrawImage(ByVal pbox As PictureBox)
        If GFX Is Nothing Then Return
        ' ------------------------------------------------------
        InitImage(pbox)
        GFX.Clear(BackColor)
        DrawScale()
        ' ------------------------------------------------------
        Dim oldx As Single = -99999
        Dim oldy As Single = -99999
        Dim oldz As Single = -99999
        Dim x, y, z As Single
        Dim xmm, ymm As Single
        Dim oldt As Long = 0
        Dim ds As Single = 2 * NeedleDiam * PixelPerMM
        Dim rs As Single = ds / 2.0F
        Dim dw As Single = WireDiam * PixelPerMM
        Dim rw As Single = dw / 2.0F
        Dim dn As Single = NeedleDiam * PixelPerMM
        Dim rn As Single = dn / 2.0F
        For i As Int32 = 0 To DataArray1.GetNumEntries - 1
            With DataArray1.GetElement(i)
                xmm = .x
                ymm = .y
                x = MMtoPixelX(xmm)
                y = MMtoPixelY(ymm)
                z = 360 * (.z Mod 1)

                ' TODO debug
                'If i >= 143 Then
                '    i = i
                'End If

                If oldx > -99999 Then
                    If .t - oldt > 30 Then
                        'GFX.DrawLine(StartPen, oldx, oldy, x, y)
                        GFX.FillEllipse(StartBrush, x - rs, y - rs, ds, ds)
                    Else
                        If IsNeedleUp(z) Then
                            If FlagTrueXYZ Then
                                GFX.FillEllipse(WireBrush, x - rw, y - rw, dw, dw)
                            Else
                                GFX.DrawLine(WirePen, oldx, oldy, x, y)
                            End If
                        Else
                            GFX.FillEllipse(NeedleBrush, x - rn, y - rn, dn, dn)
                        End If
                    End If
                    oldt = .t
                End If
                oldx = x
                oldy = y
                oldz = z
            End With
        Next
        ' ------------------------------------------------------
        If SelectedIndex >= 0 Then
            With DataArray1.GetElement(SelectedIndex)
                x = MMtoPixelX(.x)
                y = MMtoPixelY(.y)
                z = 360 * (.z Mod 1)
            End With
            With DataArray1.GetElement(SelectedIndex - 1)
                oldx = MMtoPixelX(.x)
                oldy = MMtoPixelY(.y)
                oldz = 360 * (.z Mod 1)
            End With
            If IsNeedleUp(z) Then
                If FlagTrueXYZ Then
                    GFX.FillEllipse(SelectionBrush, x - rw, y - rw, dw, dw)
                Else
                    GFX.DrawLine(SelectionPen, oldx, oldy, x, y)
                End If
            Else
                GFX.FillEllipse(Brushes.Gray, x - rn, y - rn, dn, dn)
            End If
        End If
        ' ------------------------------------------------------
    End Sub

End Module
